<?php

namespace Drupal\Tests\content_moderation_edit_notify\Functional;

use Drupal\filter\Entity\FilterFormat;
use Drupal\Tests\BrowserTestBase;
use Drupal\node\Entity\Node;
use Drupal\Tests\content_moderation\Traits\ContentModerationTestTrait;

/**
 * Test module main functionality.
 *
 * @group content_moderation_edit_notify
 */
class ContentModerationNotifyTest extends BrowserTestBase {

  use ContentModerationTestTrait;

  /**
   * Permissions to grant admin user.
   *
   * @var array
   */
  protected $permissions = [
    'view latest version',
    'view any unpublished content',
    'use editorial transition create_new_draft',
    'use editorial transition publish',
    'use editorial transition archive',
    'use editorial transition archived_draft',
    'use editorial transition archived_published',
    'use text format basic_html',
    'create page content',
    'edit any page content',
  ];

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'node',
    'filter',
    'content_moderation',
    'content_moderation_edit_notify',
  ];

  /**
   * An user with editing permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $editor_1;

  /**
   * An user with editing permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $editor_2;

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp();

    // Create a content type.
    $this
      ->drupalCreateContentType([
      'type' => 'page',
      'name' => 'Basic page',
    ]);

    // Create a workflow on this content type.
    $workflow = $this
      ->createEditorialWorkflow();
    $workflow
      ->getTypePlugin()
      ->addEntityTypeAndBundle('node', 'page');
    $workflow
      ->save();

    // Set up the filter formats used by this test.
    $basic_html_format = FilterFormat::create([
      'format' => 'basic_html',
      'name' => 'Basic HTML',
      'filters' => [
        'filter_html' => [
          'status' => 1,
          'settings' => [
            'allowed_html' => '<p> <br> <strong> <a> <em>',
          ],
        ],
      ],
    ]);
    $basic_html_format
      ->save();

    $this->editor_1 = $this
      ->drupalCreateUser($this->permissions);

    $this->editor_2 = $this
      ->drupalCreateUser($this->permissions);
  }

  /**
   * Tests the Controller.
   */
  public function testController() {
    $config = $this->config('content_moderation_edit_notify.settings');
    // Login.
    $this->drupalLogin($this->editor_1);

    // Create new moderated content in draft.
    $this->drupalPostForm('node/add/page', [
      'title[0][value]' => 'Draft page',
      'body[0][value]' => 'First version of the page.',
      'moderation_state[0][state]' => 'draft',
    ], t('Save'));
    
    $node = $this->drupalGetNodeByTitle('Draft page');
    $nid = $node->id();
    $vid_draft = $node->vid->value;

    $checkUrl = '/content_moderation_edit_notify/' . $nid . '/' . $vid_draft .'/check';
    $this->drupalGet($checkUrl);
    $this->assertSession()->responseContains("{}");

    // Update the node.
    $this->drupalPostForm('node/'. $nid .'/edit', [
      'title[0][value]' => 'Draft node rev2',
      'moderation_state[0][state]' => 'draft',
    ], t('Save'));

    $node = $this->drupalGetNodeByTitle('Draft node rev2');
    $vid_draft_edited = $node->vid->value;

    $checkUrl = '/content_moderation_edit_notify/' . $nid . '/' . $vid_draft .'/check';
    $this->drupalGet($checkUrl);
    $page = $this->getSession()->getPage()->getContent();
    $json = json_decode($page);
    $this->assertTrue($config->get('message_unpublished'), $json->message);

    // Now make it published.
    $this->drupalPostForm('node/'. $nid .'/edit', [
      'title[0][value]' => 'Published node',
      'moderation_state[0][state]' => 'published',
    ], t('Save'));

    $node = $this->drupalGetNodeByTitle('Published node');
    $vid_published = $node->vid->value;

    $checkUrl = '/content_moderation_edit_notify/' . $nid . '/' . $vid_draft_edited .'/check';
    $this->drupalGet($checkUrl);
    $page = $this->getSession()->getPage()->getContent();
    $json = json_decode($page);
    $this->assertTrue($config->get('message_published'), $json->message);
  }
}
